@@ -75,12 +75,18 @@ static char      f_szTestListFileName[MAX_FILENAME_LENGTH] = "";   /**< Current
 static char      f_szTestResultFileName[MAX_FILENAME_LENGTH] = ""; /**< Current output file name for the test results file. */
 static FILE*     f_pTestResultFile = NULL;                  /**< FILE pointer the test results file. */
 
-static CU_BOOL f_bWriting_BCUNIT_RUN_SUITE = CU_FALSE;       /**< Flag for keeping track of when a closing xml tag is required. */
+static CU_BOOL   f_bWriting_BCUNIT_RUN_SUITE = CU_FALSE;       /**< Flag for keeping track of when a closing xml tag is required. */
 
 static CU_BOOL   bJUnitXmlOutput = CU_FALSE;                /**< Flag for toggling the xml junit output or keeping the original. Off is the default */
+
+static CU_BOOL	 bPartialSuiteJUnitReport = CU_FALSE;		/** Flag for toggling englobing <testsuites> tags. Warning: allows imcomplete JUnit results file where only <testsuite></testsuite> will be present. This is to allow JUnit-xml output for a single suite */
+
 static char _gPackageName[50] = "";
 
-static time_t    f_testStartTime = 0;                       /**< Start time of current running test suite. */
+//static time_t    f_testStartTime = 0;                       /**< Start time of current running test suite. */
+
+double startTime = 0.0;
+double endTime = 0.0;
 
 /*=================================================================
  *  Static function forward declarations
@@ -235,9 +241,11 @@ static CU_ErrorCode initialize_result_file(const char* szFilename)
     setvbuf(f_pTestResultFile, NULL, _IONBF, 0);
 
     if (bJUnitXmlOutput == CU_TRUE) {
-      fprintf(f_pTestResultFile,
-              "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
-              "<testsuites> \n");
+	    if (bPartialSuiteJUnitReport == CU_FALSE) {
+		    fprintf(f_pTestResultFile,
+			    "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
+			    "<testsuites> \n");
+	    }
     } else {
       fprintf(f_pTestResultFile,
               "<?xml version=\"1.0\" ?> \n"
@@ -263,7 +271,12 @@ static void automated_test_start_message_handler(const CU_pTest pTest, const CU_
 	char *szTempName = NULL;
 	size_t szTempName_len = 0;
 
-  f_testStartTime = time(NULL);
+  //Gathering information about current time for the suite start time
+  time_t suiteStartTime = time(NULL);
+  struct tm tm = *localtime(&suiteStartTime);
+
+  //Getting the start time of the test to compute test duration
+  startTime = CU_get_wall_time();
 
   if( test_start_handler ){
     (*test_start_handler)(pTest, pSuite);
@@ -296,10 +309,18 @@ static void automated_test_start_message_handler(const CU_pTest pTest, const CU_
 
     if (bJUnitXmlOutput == CU_TRUE) {
       fprintf(f_pTestResultFile,
-              "  <testsuite errors=\"%u\" tests=\"%u\" name=\"%s\"> \n",
-              0 , /* Errors */
+              "  <testsuite name=\"%s\" tests=\"%u\" time=\"0\" failures=\"%u\" errors=\"%u\" skipped=\"0\" timestamp=\"%d-%02d-%02dT%02d:%02d:%02d\"> \n",
+              (NULL != szTempName) ? szTempName : "", /* Name */
               pSuite->uiNumberOfTests, /* Tests */
-              (NULL != szTempName) ? szTempName : ""); /* Name */
+              pSuite->uiNumberOfTestsFailed, /* Tests failure */
+              0,
+              tm.tm_year + 1900,
+              tm.tm_mon + 1,
+              tm.tm_mday,
+              tm.tm_hour,
+              tm.tm_min,
+              tm.tm_sec
+              ); /* Errors */
     } else {
       fprintf(f_pTestResultFile,
               "    <BCUNIT_RUN_SUITE> \n"
@@ -333,6 +354,9 @@ static void automated_test_complete_message_handler(const CU_pTest pTest,
   CU_pFailureRecord pTempFailure = pFailure;
   const char *pPackageName = CU_automated_package_name_get();
 
+  //Getting the end time of the test to compute test duration
+  endTime = CU_get_wall_time();
+
   if( test_complete_handler ){
     (*test_complete_handler)(pTest, pSuite, pFailure);
   }
@@ -353,19 +377,30 @@ static void automated_test_complete_message_handler(const CU_pTest pTest,
         assert((NULL != pTempFailure->pTest) && (pTempFailure->pTest == pTest));
 
         if (NULL != pTempFailure->strCondition) {
-          szTemp = (char *)CU_MALLOC(CU_translated_strlen(pTempFailure->strCondition));
-          CU_translate_special_characters(pTempFailure->strCondition, szTemp, sizeof(szTemp));
+          szTemp_len = CU_translated_strlen(pTempFailure->strCondition) + 1;
+          szTemp = (char *)CU_MALLOC(szTemp_len);
+          size_t test = CU_translate_special_characters(pTempFailure->strCondition, szTemp, szTemp_len);
         }
         else {
+          //!!!!!!
           szTemp[0] = '\0';
         }
 
-        fprintf(f_pTestResultFile, "        <testcase classname=\"%s.%s\" name=\"%s\" time=\"%ld\">\n",
+        fprintf(f_pTestResultFile, "        <testcase classname=\"%s.%s\" name=\"%s\" time=\"%f\">\n",
                 pPackageName,
                 pSuite->pName,
                 (NULL != pTest->pName) ? pTest->pName : "",
-                (long)(time(NULL) - f_testStartTime));
-        fprintf(f_pTestResultFile, "            <failure message=\"%s\" type=\"Failure\">\n", szTemp);
+                endTime - startTime
+                );
+
+        if(NULL != pTempFailure->pNext) {
+          fprintf(f_pTestResultFile, "            <failure message=\"Multiple asserts failed ...\" type=\"Failure\">\n");
+        }
+        else {
+          fprintf(f_pTestResultFile, "            <failure message=\"%s\" type=\"Failure\">\n", szTemp);
+        }
+
+
       } /* if */
     }
 
@@ -426,11 +461,12 @@ static void automated_test_complete_message_handler(const CU_pTest pTest,
   }
   else {
     if (bJUnitXmlOutput == CU_TRUE) {
-      fprintf(f_pTestResultFile,  "        <testcase classname=\"%s.%s\" name=\"%s\" time=\"%ld\"/>\n",
+      fprintf(f_pTestResultFile,  "        <testcase classname=\"%s.%s\" name=\"%s\" time=\"%f\"/>\n",
               pPackageName,
               pSuite->pName,
               (NULL != pTest->pName) ? pTest->pName : "",
-              (long)(time(NULL) - f_testStartTime));
+              endTime - startTime
+              );
     } else {
       fprintf(f_pTestResultFile,
               "        <BCUNIT_RUN_TEST_RECORD> \n"
@@ -640,17 +676,20 @@ static CU_ErrorCode uninitialize_result_file(void)
   time(&tTime);
   szTime = ctime(&tTime);
   if (szTime) szTime[24] = '\0';
-      if (bJUnitXmlOutput == CU_TRUE) {
-        fprintf(f_pTestResultFile,
-                "    </testsuite>\n</testsuites>\n");
-      }
-      else {
-        fprintf(f_pTestResultFile,
-                "  <BCUNIT_FOOTER> %s" CU_VERSION " - %s </BCUNIT_FOOTER> \n"
-                "</BCUNIT_TEST_RUN_REPORT>\n",
-                _("File Generated By BCUnit v"),
-                (NULL != szTime) ? szTime : "");
-      }
+  if (bJUnitXmlOutput == CU_TRUE) {
+	  fprintf(f_pTestResultFile,
+		  "    </testsuite>\n");
+	  if (bPartialSuiteJUnitReport == CU_FALSE) {
+		  fprintf(f_pTestResultFile, "</testsuites>\n");
+	  }
+  }
+  else {
+	  fprintf(f_pTestResultFile,
+		  "  <BCUNIT_FOOTER> %s" CU_VERSION " - %s </BCUNIT_FOOTER> \n"
+		  "</BCUNIT_TEST_RUN_REPORT>\n",
+		  _("File Generated By BCUnit v"),
+		  (NULL != szTime) ? szTime : "");
+  }
 
   if (0 != fclose(f_pTestResultFile)) {
     CU_set_error(CUE_FCLOSE_FAILED);
@@ -789,6 +828,15 @@ void CU_automated_enable_junit_xml(CU_BOOL bFlag)
 }
 /** @} */
 
+/*------------------------------------------------------------------------*/
+/** Enable or Disable the englobing XML tags for <testsuites> in the JUnit file report
+ *  This allows to create partial JUnit results file that will need to be merged.
+ */
+void CU_automated_enable_partial_junit(CU_BOOL bFlag)
+{
+  bPartialSuiteJUnitReport = bFlag;
+}
+
 /*------------------------------------------------------------------------*/
 /** Set tests suites package name
  */
